"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var jest_matcher_utils_1 = require("jest-matcher-utils");
var either_1 = require("./either/either");
var maybe_1 = require("./maybe/maybe");
var getLastCallArgument = function (mock) {
    var lastCall = mock.mock.calls[mock.mock.calls.length - 1];
    return lastCall ? lastCall[0] : undefined;
};
var getReportHeader = function (fnName, pass) { return jest_matcher_utils_1.matcherHint((pass ? '.not' : '') + "." + fnName, 'received', 'expected') + "\n"; };
var getExpectedMessage = function (values) { return jest_matcher_utils_1.printExpected(values.side + "(" + values.expected.fold() + ")"); };
var getReceivedMessage = function (values) { return "Received: " + jest_matcher_utils_1.printReceived(values.received); };
var getExpectedValueMessage = function (pass, values) {
    return "Expected value" + (pass ? ' not ' : ' ') + "to be: " + getExpectedMessage(values);
};
var getExpectedSpyMessage = function (pass, values) {
    return "Expected spy" + (pass ? ' not ' : ' ') + "to have been last called with: " + getExpectedMessage(values);
};
var getValueCheckResults = function (fnName, values, pass) { return ({
    pass: pass,
    message: function () { return getReportHeader(fnName, pass) + "\n" + getExpectedValueMessage(pass, values) + "\n" + getReceivedMessage(values); },
}); };
var getSpyCheckResults = function (fnName, values, pass) { return ({
    pass: pass,
    message: function () { return getReportHeader(fnName, pass) + "\n" + getExpectedSpyMessage(pass, values) + "\n" + getReceivedMessage(values); },
}); };
var isRight = function (values) { return !!values.received.isRight && values.received.equals(values.expected); };
var isLeft = function (values) { return !!values.received.isLeft && values.received.equals(values.expected); };
var isJust = function (values) { return !!values.received.isJust && values.received.equals(values.expected); };
var isNone = function (values) { return !!values.received.isNone && values.received.equals(values.expected); };
expect.extend({
    toBeJust: function (received, expected) {
        var values = { expected: maybe_1.just(expected), received: maybe_1.just(received), side: 'just' };
        var pass = isJust(values);
        return getValueCheckResults('toBeJust', values, pass);
    },
    toBeNone: function (received) {
        var values = { expected: maybe_1.none(), received: maybe_1.maybe(received), side: 'none' };
        var pass = isNone(values);
        return getValueCheckResults('toBeNone', values, pass);
    },
    toBeRight: function (received, expected) {
        var values = { expected: either_1.right(expected), received: either_1.right(received), side: 'right' };
        var pass = isRight(values);
        return getValueCheckResults('toBeRight', values, pass);
    },
    toBeLeft: function (received, expected) {
        var values = { expected: either_1.left(expected), received: either_1.left(received), side: 'left' };
        var pass = isLeft(values);
        return getValueCheckResults('toBeLeft', values, pass);
    },
    toHaveBeenLastCalledWithRight: function (received, expected) {
        var values = {
            expected: either_1.right(expected),
            received: either_1.right(getLastCallArgument(received)),
            side: 'right',
        };
        var pass = isRight(values);
        return getSpyCheckResults('toHaveBeenLastCalledWithRight', values, pass);
    },
    toHaveBeenLastCalledWithLeft: function (received, expected) {
        var values = { expected: either_1.left(expected), received: either_1.left(getLastCallArgument(received)), side: 'left' };
        var pass = isLeft(values);
        return getSpyCheckResults('toHaveBeenLastCalledWithLeft', values, pass);
    },
    toHaveBeenLastCalledWithJust: function (received, expected) {
        var values = {
            expected: maybe_1.just(expected),
            received: maybe_1.just(getLastCallArgument(received)),
            side: 'just',
        };
        var pass = isJust(values);
        return getSpyCheckResults('toHaveBeenLastCalledWithJust', values, pass);
    },
    toHaveBeenLastCalledWithNone: function (received) {
        var values = { expected: maybe_1.none(), received: maybe_1.maybe(received), side: 'none' };
        var pass = isNone(values);
        return getSpyCheckResults('toHaveBeenLastCalledWithNone', values, pass);
    },
});
//# sourceMappingURL=jest-matchers.js.map